package org.glassfish.bugs;

import java.io.PrintWriter;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.resource.spi.security.PasswordCredential;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

/**
 * @author cj91
 * 
 */
public class MyJCAManagedConnection implements ManagedConnection {

	private PrintWriter out;
	private MyJCAConnectionImpl myJCAConnectionImpl;
	private Set<ConnectionEventListener> listeners = Collections
		.synchronizedSet(new HashSet<ConnectionEventListener>());

	protected ConnectionRequestInfo cxRequestInfo;
	private Subject subject;

	public MyJCAManagedConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) {
		this.subject = subject;
		this.cxRequestInfo = cxRequestInfo;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public MyJCAConnectionImpl getConnection(Subject subject, ConnectionRequestInfo cxRequestInfo)
		throws ResourceException {
		String operatorId = null;
		String operatorPassword = null;

		if (subject == null) {
			subject = this.subject;
		}

		if (cxRequestInfo == null) {
			cxRequestInfo = this.cxRequestInfo;
		}

		for (PasswordCredential cred : subject.getPrivateCredentials(PasswordCredential.class)) {
			operatorId = cred.getUserName();
			operatorPassword = new String(cred.getPassword());

			if (operatorId != null && operatorPassword != null) {
				break;
			}
		}

		myJCAConnectionImpl = new MyJCAConnectionImpl(operatorId, operatorPassword);

		this.cxRequestInfo = cxRequestInfo;
		return myJCAConnectionImpl;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void destroy() throws ResourceException {
		myJCAConnectionImpl = null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void cleanup() throws ResourceException {
		myJCAConnectionImpl = null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void associateConnection(Object connection) throws ResourceException {
		this.myJCAConnectionImpl = (MyJCAConnectionImpl) connection;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void addConnectionEventListener(ConnectionEventListener listener) {
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void removeConnectionEventListener(ConnectionEventListener listener) {
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public XAResource getXAResource() throws ResourceException {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public LocalTransaction getLocalTransaction() throws ResourceException {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		return null;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setLogWriter(PrintWriter out) throws ResourceException {
		this.out = out;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public PrintWriter getLogWriter() throws ResourceException {
		return out;
	}

	/**
	 * 
	 */
	public void close() {
		ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
		for (ConnectionEventListener listener : listeners) {
			listener.connectionClosed(event);
		}
	}

	/**
	 * 
	 */
	public void error() {
		ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_ERROR_OCCURRED);
		for (ConnectionEventListener listener : listeners) {
			listener.connectionErrorOccurred(event);
		}
	}
}
